import click

from rezide.utils import interfaces
from rezide.utils import sway


def print_tree(window_manager: interfaces.TilingWindowManager) -> None:
    for node in window_manager.get_tree():
        click.echo(click.style(node.name, fg="green", bold=True))
        click.echo(f"x, y: ({node.rect.x}, {node.rect.y})")
        click.echo(f"width, height: ({node.rect.width}, {node.rect.height})")
        click.echo(f"gaps: {node.gaps}")
        click.echo("marks: ", nl=False)
        click.echo(click.style(f"{node.marks}", fg="cyan"))
        click.echo()


@click.command()
def main() -> None:
    window_manager = sway.Sway()
    print_tree(window_manager)



import collections
import logging
from typing import Iterable, Set

from rezide.utils import interfaces

# We use depth-first traversal to create each leaf node in the tree. We
# create the leftmost descendant of each parent first so that it can reserve
# space for its other siblings.


class LayoutManager(object):
    def __init__(
        self,
        config_parser: interfaces.ConfigParserInterface,
        window_manager: interfaces.TilingWindowManager,
    ) -> None:
        self._window_manager = window_manager
        # make sure that our configuration is valid
        config_parser.validate()
        tree = config_parser.get_tree()
        self._layout = Layout(tree)

    def spawn_windows(self) -> None:
        logging.debug(
            f"{self._window_manager.num_workspace_windows} windows"
            + " are open in the current workspace"
        )
        if self._window_manager.num_workspace_windows > 1:
            raise RuntimeError(
                "There are multiple windows open in the current workspace."
            )
        self._created_windows: Set[str] = set()
        for window in self._layout.zachstras_traversal():
            if window.is_parent:
                self._window_manager.split_and_mark_parent(window.data, "abc")
            elif window.data.mark in self._created_windows:
                self._window_manager.focus(window.data)
            else:
                self._window_manager.make_window(window.data)
                self._created_windows.add(window.data.mark)


class Layout(object):
    def __init__(self, tree_: interfaces.TreeNodeInterface) -> None:
        self._tree = tree_

    def zachstras_traversal(self) -> Iterable[interfaces.TreeNodeInterface]:
        node_queue = collections.deque([self._tree])
        while len(node_queue) >= 1:
            current_node = node_queue.popleft()
            logging.debug(f"dequeuing {current_node}")
            if current_node.is_parent:
                yield current_node.children[0].get_leftmost_descendant()
                yield current_node
                for child in current_node.children[1:]:
                    yield child.get_leftmost_descendant()
                for child in current_node.children:
                    node_queue.append(child)



import logging
from typing import Dict

from rezide.utils import interfaces


class ConfigParser(interfaces.ConfigParserInterface):
    """Parses a config file and creates a Tree out of it."""

    def __init__(
        self,
        config_dict: Dict,
        tree_factory: interfaces.TreeFactoryInterface,
    ) -> None:
        self._tree_factory = tree_factory
        self._layout_definitions = config_dict

    def validate(self) -> None:
        for definition_name, definition_body in self._layout_definitions.items():
            if "command" in definition_body:
                self._validate_window(definition_name, definition_body)
            elif "children" in definition_body:
                self._validate_section(definition_name, definition_body)
            else:
                logging.error(definition_body)
                raise RuntimeError(
                    f"This definition is not a Window or Section: {definition_name}"
                )

    def _validate_window(self, definition_name: str, definition_body: Dict) -> None:
        if len(definition_body) != 1:
            raise RuntimeError(f"Window must only define command: {definition_name}")

    def _validate_section(self, definition_name: str, definition_body: Dict) -> None:
        keys = set(definition_body.keys())
        if not {"split", "children", "sizes"}.issubset(keys):
            raise RuntimeError(
                f"Section must define split, children, and sizes. keys defined: {keys}"
            )
        allowed_keys = {"split", "children", "sizes", "is_layout"}
        extra_keys = keys - allowed_keys
        if len(extra_keys) > 0:
            raise RuntimeError(
                f"Section must only define these keys: {allowed_keys}. extra keys"
                + f" defined: {extra_keys}"
            )
        if sum(definition_body["sizes"]) != 100:
            raise RuntimeError(f"Sum of sizes is not 100: {definition_name}")
        for child in definition_body["children"]:
            if child not in self._layout_definitions:
                raise RuntimeError(
                    f"{child} is a child of {definition_name} but is not defined"
                )
        if len(definition_body["children"]) < 2:
            raise RuntimeError(f"{definition_name} has less than 2 children")
        if len(definition_body["children"]) != len(definition_body["sizes"]):
            raise RuntimeError(
                "The number of children to not match the number of sizes in"
                + f" {definition_name}"
            )

    def get_tree(self) -> interfaces.TreeNodeInterface:
        """Parse and validate the layout, stitch together the node definitions, and
        create a tree out of them.
        """
        layout_top_definition = self._layout_definitions["root"]
        root_node = self._construct_subtree(layout_top_definition)
        return self._tree_factory.create_tree(root_node)

    def _construct_subtree(self, subtree_dict: Dict) -> Dict:
        if "command" in subtree_dict:
            return subtree_dict.copy()
        else:
            subtree = subtree_dict.copy()
            child_subtrees = []
            for child in subtree["children"]:
                child_definition = self._layout_definitions[child]
                if "mark" not in child_definition:
                    child_definition["mark"] = child
                child_subtrees.append(self._construct_subtree(child_definition))
            subtree["children"] = child_subtrees
            return subtree



# https://stackoverflow.com/questions/36286894/name-not-defined-in-type-annotation
from __future__ import annotations

import logging
from typing import Dict, List, Optional

from rezide.utils import dtos
from rezide.utils import interfaces


class TreeFactory(interfaces.TreeFactoryInterface):
    def create_tree(self, tree_dict: Dict) -> interfaces.TreeNodeInterface:
        return self._create_subtree(tree_dict)

    def _create_subtree(self, node: Dict, parent: Optional[Section] = None) -> TreeNode:
        """Recursively create the subtree of the current node and everything below it"""
        current_node: TreeNode
        if "command" in node:
            current_node = Window(
                dtos.WindowDetails(mark=node["mark"], command=node["command"]),
                parent=parent,
            )
        elif "children" in node:
            current_node = Section(node["split"], node["sizes"], parent=parent)
            if len(node["children"]) <= 1:
                raise RuntimeError("each parent needs at least 2 children")
            for child in node["children"]:
                self._create_subtree(child, parent=current_node)
        else:
            logging.error(node)
            raise RuntimeError("invalid config file")
        return current_node


class TreeNode(interfaces.TreeNodeInterface):
    def __eq__(self, other: object) -> bool:  # pragma: nocover
        raise NotImplementedError("Can't compare base TreeNodes!")


class Section(TreeNode):
    def __init__(
        self,
        split_orientation: str,
        child_sizes: List[int],
        parent: Optional[Section] = None,
    ) -> None:
        self._split_orientation = split_orientation
        self._child_sizes = child_sizes
        self._children: List[interfaces.TreeNodeInterface] = []
        if parent:
            parent.add_child(self)

    def get_leftmost_descendant(self) -> interfaces.TreeNodeInterface:
        return self.children[0].get_leftmost_descendant()

    def add_child(self, node: interfaces.TreeNodeInterface) -> None:
        self._children.append(node)

    @property
    def is_parent(self) -> bool:
        return len(self._children) > 0

    @property
    def children(self) -> List[interfaces.TreeNodeInterface]:
        return self._children

    # probably should add coverage here eventually
    @property
    def child_sizes(self) -> List[int]:  # pragma: nocover
        return self._child_sizes

    @property
    def data(self) -> str:
        return self._split_orientation

    def __str__(self) -> str:
        return f"Section({self._children})"

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Section):
            return False
        if len(self.children) != len(other.children):
            return False
        for my_child, other_child in zip(self.children, other.children):
            if my_child != other_child:
                return False
        return self.data == other.data


class Window(TreeNode):
    def __init__(
        self, window_details: dtos.WindowDetails, parent: Optional[Section] = None
    ) -> None:
        self._window_details = window_details
        if parent:
            parent.add_child(self)

    def get_leftmost_descendant(self) -> interfaces.TreeNodeInterface:
        return self

    @property
    def children(self) -> List[interfaces.TreeNodeInterface]:  # pragma: no cover
        return []

    @property
    def is_parent(self) -> bool:
        return False

    @property
    def data(self) -> dtos.WindowDetails:
        return self._window_details

    def add_child(self, node: interfaces.TreeNodeInterface) -> None:
        raise RuntimeError(f"{self} is a Window. It should not have any children")

    def __str__(self) -> str:
        return f'Window("{self.data.mark}")'

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Window):
            return False
        return self.data == other.data



# https://stackoverflow.com/questions/36286894/name-not-defined-in-type-annotation
from __future__ import annotations

import abc
from typing import Any, Dict, List, Set

from rezide.utils import dtos


class TilingWindowManager(object):
    @abc.abstractmethod
    def make_window(self, window_details: dtos.WindowDetails) -> None:
        pass

    @abc.abstractmethod
    def resize_width(
        self, target_window: dtos.WindowDetails, section_percentage: int
    ) -> None:
        pass

    @abc.abstractmethod
    def resize_height(
        self, target_window: dtos.WindowDetails, section_percentage: int
    ) -> None:
        pass

    @abc.abstractmethod
    def focus(self, target_window: dtos.WindowDetails) -> None:
        pass

    @abc.abstractmethod
    def split_and_mark_parent(self, split_type: str, mark: str) -> None:
        pass

    @property
    @abc.abstractmethod
    def num_workspace_windows(self) -> int:
        """Count the windows on the current workspace"""
        pass

    @abc.abstractmethod
    def get_tree(self) -> List:
        pass

    @abc.abstractmethod
    def get_window_sizes(self) -> Dict:
        pass


class ConfigReader(object):
    @abc.abstractmethod
    def read(self, path: str) -> Dict:
        pass


class TileFactoryInterface(object):
    @abc.abstractmethod
    def make_tile(
        self,
        relative_width: float,
        relative_height: float,
        window_details: dtos.WindowDetails,
    ) -> dtos.Tile:
        pass


class FileStore(object):
    """Any system that could store files, like a local filesystem"""

    @abc.abstractmethod
    def path_exists(self, path: str) -> bool:
        pass

    @abc.abstractmethod
    def exists_as_dir(self, path: str) -> bool:
        pass

    @abc.abstractmethod
    def exists_as_file(self, path: str) -> bool:
        pass

    @abc.abstractmethod
    def read_file(self, path: str) -> str:
        pass

    @abc.abstractmethod
    def list_directory_contents(self, path: str) -> Set[str]:
        pass


class TreeNodeInterface(object):
    @abc.abstractmethod
    def add_child(self, node: TreeNodeInterface) -> None:
        pass

    @abc.abstractmethod
    def get_leftmost_descendant(self) -> TreeNodeInterface:
        pass

    @property
    @abc.abstractmethod
    def is_parent(self) -> bool:
        pass

    @property
    @abc.abstractmethod
    def children(self) -> List[TreeNodeInterface]:
        pass

    @property
    @abc.abstractmethod
    def data(self) -> Any:
        pass


class TreeFactoryInterface(object):
    @abc.abstractmethod
    def create_tree(self, root_node: Dict) -> TreeNodeInterface:
        pass


class ConfigParserInterface(object):
    @abc.abstractmethod
    def validate(self) -> None:
        pass

    @abc.abstractmethod
    def get_tree(self) -> TreeNodeInterface:
        pass


class ConfigDir(object):
    @abc.abstractmethod
    def list_layouts(self) -> Set[str]:
        pass

    @abc.abstractmethod
    def get_layout_file_path(self, layout_name: str) -> str:
        pass



from typing import Any, NamedTuple

# data-transfer objects (DTOs)
# objects that don't have much functionality besides storing
# useful collections of information


class ScreenDimensions(NamedTuple):
    width: int
    height: int


class Window(NamedTuple):
    command: str
    width: int
    height: int
    mark: str


class WindowDetails(NamedTuple):
    mark: str
    command: str


# todo: remove tiles now that we don't use them anymore
class Tile(NamedTuple):
    """A class that represents the area covered by a window and its gaps.

    If you combine all tiles in a layout, it should cover the entire screen
    (excluding the status bar???)

    A tile's main job is to store a Window class and the dimensions of the tile
    """

    width: int
    height: int
    window: Window


class Env(NamedTuple):
    """Environment variables"""

    home: str
    xdg_config_home: str


class WindowManagerCall(NamedTuple):
    """Used for verifying calls to a window manager"""

    command: str
    arg: Any



import math

from rezide.utils import dtos
from rezide.utils import interfaces


class TileFactory(interfaces.TileFactoryInterface):
    def __init__(self, screen_dimensions: dtos.ScreenDimensions) -> None:
        self._screen_dimensions = screen_dimensions

    def make_tile(
        self,
        relative_width: float,
        relative_height: float,
        window_details: dtos.WindowDetails,
    ) -> dtos.Tile:
        # calculate the pixel width and height of the window and tile
        absolute_width = math.floor(relative_width * self._screen_dimensions.width)
        absolute_height = math.floor(relative_height * self._screen_dimensions.height)
        window = dtos.Window(
            command=window_details.command,
            width=absolute_width,
            height=absolute_height,
            mark=window_details.mark,
        )
        return dtos.Tile(width=absolute_width, height=absolute_height, window=window)



import pprint

import click

from rezide.utils import interfaces
from rezide.utils import sway


@click.command()
def main() -> None:
    window_manager = sway.Sway()
    print_window_sizes(window_manager)


def print_window_sizes(window_manager: interfaces.TilingWindowManager) -> None:
    pprint.pprint(window_manager.get_window_sizes())



import logging
import time
from typing import Dict, List, Tuple

import i3ipc

from rezide.utils import dtos
from rezide.utils import interfaces

"""We need to sleep for a short time since processes take time to start.
If we don't sleep, then we may be focusing on a different window by the
time that the current window is spawning, which would put it in the wrong
split.
"""

# WINDOW_MARK_EVENT = i3ipc.Event.WINDOW_MARK
# WINDOW_FOCUS_EVENT = i3ipc.Event.WINDOW_FOCUS
NEW_WINDOW_EVENT = i3ipc.Event.WINDOW_NEW

# marking and focusing are such quick actions that we can't just fire and wait.
# python isn't fast enough to catch up to them!
# TODO: set up something to expect an event and wait instead of manually waiting
MARK_SLEEP_TIME = 0
FOCUS_SLEEP_TIME = 0

# there doesn't seem to be either of these events
RESIZE_SLEEP_TIME = 0.25
SPLIT_SLEEP_TIME = 0.25

# TODO: add logging for commands


class Sway(interfaces.TilingWindowManager):
    def __init__(self) -> None:
        self._sway = i3ipc.Connection()

    def sleep_until_event(self, event: i3ipc.events.IpcBaseEvent) -> None:
        """Sleep until a certain IPC event is detected"""

        def wake_up(
            connection: i3ipc.connection.Connection, event: i3ipc.events.IpcBaseEvent
        ) -> None:
            """Callback function that ends the sleep"""
            logging.debug(f"detected {event.change} event")
            connection.main_quit()

        # set up the wakeup callback
        self._sway.on(event, wake_up)
        logging.debug(f"waiting for {event} event")
        # start the main loop, effectively sleeping until the wakeup callback is triggered
        self._sway.main()
        # unsubscribe the handler so we don't have anything hanging around
        self._sway.off(wake_up)

    def make_window(self, window_details: dtos.WindowDetails) -> None:
        """Create a window then mark it"""
        logging.debug(f"creating window with command {window_details.command}")
        self._sway.command(f"exec {window_details.command}")
        self.sleep_until_event(NEW_WINDOW_EVENT)
        logging.debug(f"marking window with mark {window_details.mark}")
        self._get_focused_window().command(f"mark {window_details.mark}")
        time.sleep(MARK_SLEEP_TIME)

    def focus(self, target_window: dtos.WindowDetails) -> i3ipc.Con:
        logging.debug(f"focusing window with mark {target_window.mark}")
        window = self._get_window(target_window.mark)
        window.command("focus")
        time.sleep(FOCUS_SLEEP_TIME)
        return window

    def split_and_mark_parent(self, split_type: str, mark: str) -> None:
        logging.debug(
            f"splitting the parent of the focused node and marking the container with {mark}"
        )
        focused = self._get_focused_window()
        if split_type == "vertical":
            focused.command("split vertical")
        elif split_type == "horizontal":
            focused.command("split horizontal")
        else:
            raise RuntimeError(f"invalid split type: {split_type}")
        time.sleep(SPLIT_SLEEP_TIME)
        focused.command("focus parent")
        time.sleep(FOCUS_SLEEP_TIME)
        parent = self._get_focused_window()
        parent.command(f"mark {mark}")
        time.sleep(MARK_SLEEP_TIME)
        # need to give focus back to the window that we just focused
        focused.command("focus")
        time.sleep(FOCUS_SLEEP_TIME)

    def resize_width(
        self, target_window: dtos.WindowDetails, section_percentage: int
    ) -> None:
        window = self.focus(target_window)
        time.sleep(FOCUS_SLEEP_TIME)
        window.command(f"resize set width {section_percentage} ppt")

    def resize_height(
        self, target_window: dtos.WindowDetails, section_percentage: int
    ) -> None:
        window = self.focus(target_window)
        time.sleep(FOCUS_SLEEP_TIME)
        window.command(f"resize set height {section_percentage} ppt")

    def get_window_sizes(self) -> Dict[Tuple, Dict[str, float]]:
        return {
            tuple(window.marks): {
                "width": window.window_rect.width,
                "height": window.window_rect.height,
            }
            for window in self._get_windows_in_current_workspace()
        }

    def get_tree(self) -> List:
        current_workspace = self._get_focused_window().workspace()
        return current_workspace.descendants()

    def _get_focused_window(self) -> i3ipc.Con:
        tree = self._sway.get_tree()
        focused = tree.find_focused()
        if not focused:
            raise RuntimeError("There is no focused window")
        return focused

    def _get_window(self, mark: str) -> i3ipc.Con:
        tree = self._sway.get_tree()
        windows = tree.find_marked(mark)
        logging.debug(f'searching for mark "{mark}"')
        logging.debug(f"windows are {windows}")
        if len(windows) > 1:
            raise RuntimeError(
                f'There is more than 1 window that matches the regex "{mark}"'
            )
        if len(windows) < 1:
            raise RuntimeError(f'There are no windows that matches the regex "{mark}"')
        return windows[0]

    def _get_windows_in_current_workspace(self) -> List[i3ipc.Con]:
        current_workspace_num = self._get_current_workspace_num()
        windows_in_current_workspace = []
        for section in self._sway.get_tree().leaves():
            logging.debug(
                f'"{section.name}" is in workspace {section.workspace().num} with '
                + f'marks "{section.marks}"'
            )
            if section.workspace().num == current_workspace_num:
                windows_in_current_workspace.append(section)
        return windows_in_current_workspace

    def _get_current_workspace_num(self) -> i3ipc.Con:
        workspaces = self._sway.get_workspaces()
        for workspace in workspaces:
            if workspace.focused:
                if workspace.num == -1:
                    raise RuntimeError("The current workspace is a named workspace")
                return workspace.num
        else:
            raise RuntimeError("There is no current workspace")

    @property
    def num_workspace_windows(self) -> int:
        """Get the number of windows open on the current workspace

        The current workspace must not be a "named workspace"
        https://i3ipc-python.readthedocs.io/en/latest/replies.html#i3ipc.WorkspaceReply
        """
        return len(self._get_windows_in_current_workspace())



import logging
import os
from typing import Optional, Set

from rezide.utils import dtos
from rezide.utils import interfaces


class ConfigDir(interfaces.ConfigDir):
    """Finds and exposes operations for a rezide configuration directory"""

    def __init__(
        self,
        filestore: interfaces.FileStore,
        env: dtos.Env,
        specified_dir: Optional[str] = None,
    ) -> None:
        self._filestore = filestore
        self._select_config_dir(env, specified_dir)

    def _select_config_dir(self, env: dtos.Env, specified_dir: Optional[str]) -> None:
        """Search through options for configuration directories in this order:
        1. directory that has been manually specified by the user
        2. $XDG_CONFIG_HOME/rezide
        3. $HOME/.rezide
        """
        xdg_config_dir = os.path.join(env.xdg_config_home, "rezide")
        home_config_dir = os.path.join(env.home, ".rezide")

        if specified_dir:
            if not self._filestore.exists_as_dir(specified_dir):
                raise RuntimeError(f"{specified_dir} does not exist")
            self._dir = specified_dir
        elif self._filestore.exists_as_dir(xdg_config_dir):
            self._dir = xdg_config_dir
        elif self._filestore.exists_as_dir(home_config_dir):
            self._dir = home_config_dir
        else:
            raise RuntimeError(
                f"Failed to find config dir. looked in '{xdg_config_dir}' and"
                + f" '{home_config_dir}'"
            )
        logging.info(f"reading from '{self._dir}' as config dir")

    def list_layouts(self) -> Set[str]:
        """List all available layouts in the config directory"""
        logging.info(f"listing layouts in {self._dir}")
        layouts = set()
        for file_or_dir in self._filestore.list_directory_contents(self._dir):
            logging.debug(f"examining {file_or_dir} to see if it has a config")
            absolute_path_to_dir = os.path.join(self._dir, file_or_dir)
            absolute_path_to_config_file = os.path.join(
                absolute_path_to_dir, "config.toml"
            )
            is_dir = self._filestore.exists_as_dir(absolute_path_to_dir)
            has_config_file = self._filestore.exists_as_file(
                absolute_path_to_config_file
            )
            if is_dir and has_config_file:
                layouts.add(file_or_dir)
        return layouts

    def get_layout_file_path(self, layout_name: str) -> str:
        """Given a name of a layout, check for a matching toml file in the config directory
        and return its absolute file path if it exists
        """
        file_path = os.path.join(self._dir, layout_name, "config.toml")
        if not self._filestore.exists_as_file(file_path):
            raise RuntimeError(f"Layout '{layout_name}' doesn't exist in '{self._dir}'")
        return file_path



from typing import Dict

import toml

from rezide.utils import interfaces


class TomlReader(interfaces.ConfigReader):
    def __init__(self, filestore: interfaces.FileStore) -> None:
        self._filestore = filestore

    def read(self, path: str) -> Dict:
        """Convert a TOML file at `path` to a python Dict"""
        toml_str = self._filestore.read_file(path)
        return dict(toml.loads(toml_str))



import logging
import sys
from typing import Any, Dict

import click

import rezide
from rezide.utils import config_dir
from rezide.utils import config_parser
from rezide.utils import config_readers
from rezide.utils import dtos
from rezide.utils import filestore
from rezide.utils import layouts
from rezide.utils import sway
from rezide.utils import tree

# maps from verbosity level to log levels
VERBOSITY_LOG_LEVELS = {
    0: logging.WARNING,
    1: logging.INFO,
    2: logging.DEBUG,
}


# TODO: is it possible to do a dry-run where we just log info but don't
# open windows or run commands? maybe a fake supbrocess runner?


@click.group()
@click.option(
    "-v",
    "--verbose",
    "verbosity_level",
    default=0,
    count=True,
    help="Set verbosity. Add more v's to increase verbosity. For example, -v is "
    + "verbosity level 1 and -vv is verbosity level 2",
)
@click.option(
    "-c",
    "--xdg-config-home-dir",
    envvar="XDG_CONFIG_HOME",
    help="The directory for your XDG config files. Reads from the XDG_CONFIG_HOME"
    + " environment variable by default.",
)
@click.option(
    "--user-home-dir",
    envvar="HOME",
    help="The current user's home directory. Reads from the HOME environment variable"
    + " by default.",
)
@click.pass_context
@click.version_option(version=rezide.__version__)
def main(
    context: click.Context,
    verbosity_level: int,
    xdg_config_home_dir: str,
    user_home_dir: str,
) -> None:
    """todo: write me"""
    # ensure that ctx.obj exists and is a dict (in case `cli()` is called outside of
    # the python entrypoint)
    context.ensure_object(dict)
    # cast the object to Dict and then ignore the error that tells us we can't
    # cast to Dict. We totally can! We just don't control the `click` source code,
    # so `context.obj` will always be typed as Optional[Any] even if we make sure that
    # it's a Dict :(
    context.obj: Dict[str, Any]  # type: ignore[misc]

    log_level = VERBOSITY_LOG_LEVELS[verbosity_level]
    logging.basicConfig(level=log_level)
    logging.info(f"Log level set to {log_level}")
    sys.tracebacklimit = verbosity_level
    env = dtos.Env(home=user_home_dir, xdg_config_home=xdg_config_home_dir)
    context.obj["env"] = env
    context.obj["config_dir"] = config_dir.ConfigDir(filestore.LocalFilestore(), env)


@main.command()
@click.argument("layout_name")
@click.pass_context
def open(context: click.Context, layout_name: str) -> None:
    """Open the IDE of your choice"""
    context.obj: Dict[str, Any]  # type: ignore[misc]
    config_directory = context.obj["config_dir"]
    config_file_path = config_directory.get_layout_file_path(layout_name)
    config_reader = config_readers.TomlReader(filestore.LocalFilestore())
    config_dict = config_reader.read(config_file_path)
    parser = config_parser.ConfigParser(config_dict, tree.TreeFactory())
    window_manager = sway.Sway()
    layout = layouts.LayoutManager(parser, window_manager)
    application = Rezide(context.obj["env"], layout)
    application.run(layout_name)


# I want to handle this with an "eager option", but we wouldn't be able to retrieve the
# context from the environment variables without writing a lot more custom code
# so it makes more sense just to use groups and subcommands
#
# https://click.palletsprojects.com/en/8.0.x/options/?highlight=eager#callbacks-and-eager-options
@main.command()
@click.pass_context
def list_layouts(context: click.Context) -> None:
    """List all available layouts"""
    context.obj: Dict[str, Any]  # type: ignore[misc]
    config_directory: config_dir.ConfigDir = context.obj["config_dir"]
    for layout in config_directory.list_layouts():
        click.secho(layout, fg="blue")


class Rezide(object):
    """Manages the application's state and calls the appropriate functions"""

    def __init__(
        self,
        env: dtos.Env,
        layout: layouts.LayoutManager,
    ) -> None:
        self._layout = layout
        logging.debug(f"Env is {env}")

    def run(self, layout_name: str) -> None:
        self._layout.spawn_windows()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['rezide', 'rezide.utils']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.0,<9.0.0', 'i3ipc>=2.2.1,<3.0.0', 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['get-tree = rezide.get_tree:main',
                     'get-window-sizes = rezide.get_window_sizes:main',
                     'rzd = rezide.rezide:main']}

setup_kwargs = {
    'name': 'rezide',
    'version': '0.10.0',
    'description': '',
    'long_description': None,
    'author': 'abstractlyZach',
    'author_email': 'zach3lee@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import os
from typing import Set

from rezide.utils import interfaces


class LocalFilestore(interfaces.FileStore):
    def path_exists(self, path: str) -> bool:
        return os.path.exists(path)

    def read_file(self, path: str) -> str:
        with open(path, "r") as infile:
            return infile.read()

    def exists_as_dir(self, path: str) -> bool:
        return self.path_exists(path) and os.path.isdir(path)

    def exists_as_file(self, path: str) -> bool:
        return self.path_exists(path) and os.path.isfile(path)

    def list_directory_contents(self, path: str) -> Set[str]:
        if self.exists_as_dir(path):
            return set(os.listdir(path))
        raise RuntimeError(f"{path} is not a valid directory")
